@@ -1,10 +1,12 @@
 """Aliases for the xonsh shell."""
+
 import argparse
 import collections.abc as cabc
 import functools
 import inspect
 import os
 import re
+import shutil
 import sys
 import types
 import typing as tp
@@ -30,6 +32,7 @@ from xonsh.platform import (
     ON_OPENBSD,
     ON_WINDOWS,
 )
+from xonsh.procs.specs import SpecAttrModifierAlias, SpecModifierAlias
 from xonsh.timings import timeit_alias
 from xonsh.tools import (
     ALIAS_KWARG_NAMES,
@@ -52,6 +55,37 @@ def EXEC_ALIAS_RE():
     return re.compile(r"@\(|\$\(|!\(|\$\[|!\[|\&\&|\|\||\s+and\s+|\s+or\s+|[>|<]")
 
 
+class FuncAlias:
+    """Provides a callable alias for xonsh commands."""
+
+    attributes_show = ["__xonsh_threadable__", "__xonsh_capturable__"]
+    attributes_inherit = attributes_show + ["__doc__"]
+
+    def __init__(self, name, func=None):
+        self.__name__ = self.name = name
+        self.func = func
+        for attr in self.attributes_inherit:
+            if (val := getattr(func, attr, None)) is not None:
+                self.__setattr__(attr, val)
+
+    def __repr__(self):
+        r = {"name": self.name, "func": self.func.__name__}
+        r |= {
+            attr: val
+            for attr in self.attributes_show
+            if (val := getattr(self, attr, None)) is not None
+        }
+        return f"FuncAlias({repr(r)})"
+
+    def __call__(
+        self, args=None, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        func_args = [args, stdin, stdout, stderr, spec, stack][
+            : len(inspect.signature(self.func).parameters)
+        ]
+        return self.func(*func_args)
+
+
 class Aliases(cabc.MutableMapping):
     """Represents a location to hold and look up aliases."""
 
@@ -84,8 +118,7 @@ class Aliases(cabc.MutableMapping):
     @tp.overload
     def register(
         self, name: str, *, dash_case: bool = True
-    ) -> tp.Callable[[types.FunctionType], types.FunctionType]:
-        ...
+    ) -> tp.Callable[[types.FunctionType], types.FunctionType]: ...
 
     def register(self, func_or_name, name=None, dash_case=True):
         """Decorator to register the given function by name."""
@@ -98,7 +131,7 @@ class Aliases(cabc.MutableMapping):
 
         return wrapper
 
-    def get(self, key, default=None):
+    def get(self, key, default=None, spec_modifiers=None):
         """Returns the (possibly modified) value. If the key is not present,
         then `default` is returned.
         If the value is callable, it is returned without modification. If it
@@ -106,16 +139,21 @@ class Aliases(cabc.MutableMapping):
         other aliases, resulting in a new list or a "partially applied"
         callable.
         """
+        spec_modifiers = spec_modifiers if spec_modifiers is not None else []
         val = self._raw.get(key)
         if val is None:
             return default
         elif isinstance(val, cabc.Iterable) or callable(val):
-            return self.eval_alias(val, seen_tokens={key})
+            return self.eval_alias(
+                val, seen_tokens={key}, spec_modifiers=spec_modifiers
+            )
         else:
             msg = "alias of {!r} has an inappropriate type: {!r}"
             raise TypeError(msg.format(key, val))
 
-    def eval_alias(self, value, seen_tokens=frozenset(), acc_args=()):
+    def eval_alias(
+        self, value, seen_tokens=frozenset(), acc_args=(), spec_modifiers=None
+    ):
         """
         "Evaluates" the alias ``value``, by recursively looking up the leftmost
         token and "expanding" if it's also an alias.
@@ -126,8 +164,18 @@ class Aliases(cabc.MutableMapping):
         callable.  The resulting callable will be "partially applied" with
         ``["-al", "arg"]``.
         """
+        spec_modifiers = spec_modifiers if spec_modifiers is not None else []
         # Beware of mutability: default values for keyword args are evaluated
         # only once.
+        if (
+            isinstance(value, cabc.Iterable)
+            and hasattr(value, "__len__")
+            and len(value) > 1
+            and (isinstance(mod := self._raw.get(str(value[0])), SpecModifierAlias))
+        ):
+            spec_modifiers.append(mod)
+            value = value[1:]
+
         if callable(value):
             return partial_eval_alias(value, acc_args=acc_args)
         else:
@@ -144,7 +192,12 @@ class Aliases(cabc.MutableMapping):
             else:
                 seen_tokens = seen_tokens | {token}
                 acc_args = rest + list(acc_args)
-                return self.eval_alias(self._raw[token], seen_tokens, acc_args)
+                return self.eval_alias(
+                    self._raw[token],
+                    seen_tokens,
+                    acc_args,
+                    spec_modifiers=spec_modifiers,
+                )
 
     def expand_alias(self, line: str, cursor_index: int) -> str:
         """Expands any aliases present in line if alias does not point to a
@@ -181,6 +234,8 @@ class Aliases(cabc.MutableMapping):
             else:
                 # need to exec alias
                 self._raw[key] = ExecAlias(val, filename=f)
+        elif isinstance(val, types.FunctionType):
+            self._raw[key] = FuncAlias(key, val)
         else:
             self._raw[key] = val
 
@@ -218,15 +273,13 @@ class Aliases(cabc.MutableMapping):
         return str(self._raw)
 
     def __repr__(self):
-        return "{}.{}({})".format(
-            self.__class__.__module__, self.__class__.__name__, self._raw
-        )
+        return f"{self.__class__.__module__}.{self.__class__.__name__}({self._raw})"
 
     _repr_pretty_ = to_repr_pretty_
 
 
 class ExecAlias:
-    """Provides a callable alias for xonsh source code."""
+    """Provides an exec alias for xonsh source code."""
 
     def __init__(self, src, filename="<exec-alias>"):
         """
@@ -286,9 +339,7 @@ class PartialEvalAliasBase:
         return self.f(args, stdin, stdout, stderr, spec, stack)
 
     def __repr__(self):
-        return "{name}({f!r}, acc_args={acc_args!r})".format(
-            name=self.__class__.__name__, f=self.f, acc_args=self.acc_args
-        )
+        return f"{self.__class__.__name__}({self.f!r}, acc_args={self.acc_args!r})"
 
 
 class PartialEvalAlias0(PartialEvalAliasBase):
@@ -408,8 +459,8 @@ def xonsh_reset(args, stdin=None):
 
 def source_foreign_fn(
     shell: str,
-    files_or_code: Annotated[tp.List[str], Arg(nargs="+")],
-    interactive=True,
+    files_or_code: Annotated[list[str], Arg(nargs="+")],
+    interactive=False,
     login=False,
     envcmd=None,
     aliascmd=None,
@@ -436,7 +487,7 @@ def source_foreign_fn(
         Name or path to the foreign shell
     files_or_code
         file paths to source or code in the target language.
-    interactive : -n, --non-interactive
+    interactive : -i, --interactive
         whether the sourced shell should be interactive
     login : -l, --login
         whether the sourced shell should be login
@@ -480,7 +531,7 @@ def source_foreign_fn(
         if not suppress_skip_message
         else suppress_skip_message
     )
-    files: tp.Tuple[str, ...] = ()
+    files: tuple[str, ...] = ()
     if prevcmd:
         pass  # don't change prevcmd if given explicitly
     elif os.path.isfile(files_or_code[0]):
@@ -543,14 +594,28 @@ def source_foreign_fn(
             msg = (
                 "Skipping application of {0!r} alias from {1!r} "
                 "since it shares a name with an existing xonsh alias. "
-                'Use "--overwrite-alias" option to apply it anyway.'
+                'Use "--overwrite-alias" option to apply it anyway. '
                 'You may prevent this message with "--suppress-skip-message" or '
                 '"$FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE = True".'
             )
             print(msg.format(k, shell), file=_stderr)
 
 
-source_foreign = ArgParserAlias(
+class SourceForeignAlias(ArgParserAlias):
+    def build(self):
+        parser = self.create_parser(**self.kwargs)
+        # for backwards compatibility
+        parser.add_argument(
+            "-n",
+            "--non-interactive",
+            action="store_false",
+            dest="interactive",
+            help="Deprecated: The default mode runs in non-interactive mode.",
+        )
+        return parser
+
+
+source_foreign = SourceForeignAlias(
     func=source_foreign_fn, has_args=True, prog="source-foreign"
 )
 
@@ -606,7 +671,7 @@ def source_alias(args, stdin=None):
 
 
 def source_cmd_fn(
-    files: Annotated[tp.List[str], Arg(nargs="+")],
+    files: Annotated[list[str], Arg(nargs="+")],
     login=False,
     aliascmd=None,
     extra_args="",
@@ -689,7 +754,7 @@ source_cmd = ArgParserAlias(func=source_cmd_fn, has_args=True, prog="source-cmd"
 
 
 def xexec_fn(
-    command: Annotated[tp.List[str], Arg(nargs=argparse.REMAINDER)],
+    command: Annotated[list[str], Arg(nargs=argparse.REMAINDER)],
     login=False,
     clean=False,
     name="",
@@ -750,7 +815,7 @@ def xexec_fn(
     except FileNotFoundError as e:
         return (
             None,
-            "xonsh: exec: file not found: {}: {}" "\n".format(e.args[1], command[0]),
+            f"xonsh: exec: file not found: {e.args[1]}: {command[0]}" "\n",
             1,
         )
 
@@ -811,9 +876,16 @@ def detect_xpip_alias():
 
     basecmd = [sys.executable, "-m", "pip"]
     try:
-        if ON_WINDOWS or IN_APPIMAGE:
+        if ON_WINDOWS:
             # XXX: Does windows have an installation mode that requires UAC?
             return basecmd
+        elif IN_APPIMAGE:
+            # In AppImage `sys.executable` is equal to path to xonsh.AppImage file and the real python executable is in $_
+            return [
+                XSH.env.get("_", "APPIMAGE_PYTHON_EXECUTABLE_NOT_FOUND"),
+                "-m",
+                "pip",
+            ]
         elif not os.access(os.path.dirname(sys.executable), os.W_OK):
             return (
                 sys.executable
@@ -843,12 +915,12 @@ def make_default_aliases():
         "exec": xexec,
         "xexec": xexec,
         "source": source_alias,
-        "source-zsh": ArgParserAlias(
+        "source-zsh": SourceForeignAlias(
             func=functools.partial(source_foreign_fn, "zsh", sourcer="source"),
             has_args=True,
             prog="source-zsh",
         ),
-        "source-bash": ArgParserAlias(
+        "source-bash": SourceForeignAlias(
             func=functools.partial(source_foreign_fn, "bash", sourcer="source"),
             has_args=True,
             prog="source-bash",
@@ -867,6 +939,14 @@ def make_default_aliases():
         "completer": xca.completer_alias,
         "xpip": detect_xpip_alias(),
         "xonsh-reset": xonsh_reset,
+        "xthread": SpecAttrModifierAlias(
+            {"threadable": True, "force_threadable": True},
+            "Mark current command as threadable.",
+        ),
+        "xunthread": SpecAttrModifierAlias(
+            {"threadable": False, "force_threadable": False},
+            "Mark current command as unthreadable.",
+        ),
     }
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
@@ -890,7 +970,7 @@ def make_default_aliases():
             "vol",
         }
         for alias in windows_cmd_aliases:
-            default_aliases[alias] = ["cmd", "/c", alias]
+            default_aliases[alias] = [os.getenv("COMSPEC"), "/c", alias]
         default_aliases["call"] = ["source-cmd"]
         default_aliases["source-bat"] = ["source-cmd"]
         default_aliases["clear"] = "cls"
@@ -898,7 +978,8 @@ def make_default_aliases():
             # Add aliases specific to the Anaconda python distribution.
             default_aliases["activate"] = ["source-cmd", "activate.bat"]
             default_aliases["deactivate"] = ["source-cmd", "deactivate.bat"]
-        if not locate_binary("sudo"):
+        if shutil.which("sudo", path=XSH.env.get_detyped("PATH")):
+            # XSH.commands_cache is not available during setup
             import xonsh.winutils as winutils
 
             def sudo(args):
