@@ -53,15 +53,20 @@ def resetting_signal_handle(sig, f):
     """Sets a new signal handle that will automatically restore the old value
     once the new handle is finished.
     """
-    oldh = signal.getsignal(sig)
+    prev_signal_handler = signal.getsignal(sig)
 
-    def newh(s=None, frame=None):
+    def new_signal_handler(s=None, frame=None):
         f(s, frame)
-        signal.signal(sig, oldh)
+        signal.signal(sig, prev_signal_handler)
         if sig != 0:
+            """
+            There is no immediate exiting here.
+            The ``sys.exit()`` function raises a ``SystemExit`` exception.
+            This exception must be caught and processed in the upstream code.
+            """
             sys.exit(sig)
 
-    signal.signal(sig, newh)
+    signal.signal(sig, new_signal_handler)
 
 
 def helper(x, name=""):
@@ -162,7 +167,7 @@ def pathsearch(func, s, pymode=False, pathobj=False):
 
 def subproc_captured_stdout(*cmds, envs=None):
     """Runs a subprocess, capturing the output. Returns the stdout
-    that was produced as a str.
+    that was produced as a str or list based on ``$XONSH_SUBPROC_OUTPUT_FORMAT``.
     """
 
     import xonsh.procs.specs
@@ -561,6 +566,7 @@ class XonshSession:
         self._completers = None
         self.builtins = None
         self._initial_builtin_names = None
+        self.last = None  # Last executed CommandPipeline.
 
     @property
     def aliases(self):
@@ -593,7 +599,7 @@ class XonshSession:
         if self._py_quit is not None:
             builtins.quit = self._py_quit
 
-    def load(self, execer=None, ctx=None, **kwargs):
+    def load(self, execer=None, ctx=None, inherit_env=True, **kwargs):
         """Loads the session with default values.
 
         Parameters
@@ -602,6 +608,10 @@ class XonshSession:
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
             Context to start xonsh session with.
+        inherit_env : bool
+            If ``True``: inherit environment variables from ``os.environ``.
+            If ``False``: use default values for environment variables and
+            set ``$XONSH_ENV_INHERITED = False``.
         """
         from xonsh.commands_cache import CommandsCache
         from xonsh.environ import Env, default_env
@@ -611,7 +621,12 @@ class XonshSession:
         if ctx is not None:
             self.ctx = ctx
 
-        self.env = kwargs.pop("env") if "env" in kwargs else Env(default_env())
+        if "env" in kwargs:
+            self.env = kwargs.pop("env")
+        elif inherit_env:
+            self.env = Env(default_env())
+        else:
+            self.env = Env({"XONSH_ENV_INHERITED": False})
 
         self.exit = False
         self.stdout_uncaptured = None
@@ -677,6 +692,10 @@ class XonshSession:
 
         if not self.builtins_loaded:
             return
+
+        if self.history is not None:
+            self.history.flush(at_exit=True)
+
         self.unlink_builtins()
         delattr(builtins, "__xonsh__")
         self.builtins_loaded = False
