@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Tools for inspecting Python objects.
 
 This file was forked from the IPython project:
@@ -8,23 +7,21 @@ This file was forked from the IPython project:
 * Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
 * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
 """
-import os
-import io
-import sys
-import types
+
 import inspect
 import itertools
 import linecache
-import collections
+import os
+import sys
+import types
 
 from xonsh.lazyasd import LazyObject
-from xonsh.tokenize import detect_encoding
+from xonsh.lazyimps import pyghooks, pygments
 from xonsh.openpy import read_py_file
-from xonsh.tools import cast_unicode, safe_hasattr, indent, print_color, format_color
-from xonsh.platform import HAS_PYGMENTS, PYTHON_VERSION_INFO
-from xonsh.lazyimps import pygments, pyghooks
+from xonsh.platform import HAS_PYGMENTS
 from xonsh.style_tools import partial_color_tokenize
-
+from xonsh.tokenize import detect_encoding
+from xonsh.tools import cast_unicode, format_color, indent, print_color, safe_hasattr
 
 # builtin docstrings to ignore
 _func_call_docstring = LazyObject(
@@ -105,7 +102,7 @@ def get_encoding(obj):
         # Print only text files, not extension binaries.  Note that
         # getsourcelines returns lineno with 1-offset and page() uses
         # 0-offset, so we must adjust.
-        with io.open(ofile, "rb") as buf:  # Tweaked to use io.open for Python 2
+        with open(ofile, "rb") as buf:  # Tweaked to use io.open for Python 2
             encoding, _ = detect_encoding(buf.readline)
         return encoding
 
@@ -209,23 +206,21 @@ def call_tip(oinfo, format_call=True):
     Parameters
     ----------
     oinfo : dict
-
     format_call : bool, optional
-      If True, the call line is formatted and returned as a string.  If not, a
-      tuple of (name, argspec) is returned.
+        If True, the call line is formatted and returned as a string.  If not, a
+        tuple of (name, argspec) is returned.
 
     Returns
     -------
     call_info : None, str or (str, dict) tuple.
-      When format_call is True, the whole call information is formatted as a
-      single string.  Otherwise, the object's name and its argspec dict are
-      returned.  If no call information is available, None is returned.
-
+        When format_call is True, the whole call information is formatted as a
+        single string.  Otherwise, the object's name and its argspec dict are
+        returned.  If no call information is available, None is returned.
     docstring : str or None
-      The most relevant docstring for calling purposes is returned, if
-      available.  The priority is: call docstring for callable instances, then
-      constructor docstring for classes, then main object's docstring otherwise
-      (regular functions).
+        The most relevant docstring for calling purposes is returned, if
+        available.  The priority is: call docstring for callable instances, then
+        constructor docstring for classes, then main object's docstring otherwise
+        (regular functions).
     """
     # Get call definition
     argspec = oinfo.get("argspec")
@@ -270,7 +265,7 @@ def find_file(obj):
     Returns
     -------
     fname : str
-      The absolute path to the file where the object was defined.
+        The absolute path to the file where the object was defined.
     """
     # get source if obj was decorated with @decorator
     if safe_hasattr(obj, "__wrapped__"):
@@ -288,7 +283,7 @@ def find_file(obj):
             except TypeError:
                 # Can happen for builtins
                 pass
-    except:  # pylint:disable=bare-except
+    except Exception:
         pass
     return cast_unicode(fname)
 
@@ -307,7 +302,7 @@ def find_source_lines(obj):
     Returns
     -------
     lineno : int
-      The line number where the object definition starts.
+        The line number where the object definition starts.
     """
     # get source if obj was decorated with @decorator
     if safe_hasattr(obj, "__wrapped__"):
@@ -322,28 +317,13 @@ def find_source_lines(obj):
                 lineno = inspect.getsourcelines(obj.__class__)[1]
             else:
                 lineno = None
-    except:  # pylint:disable=bare-except
+    except Exception:
         return None
 
     return lineno
 
 
-if PYTHON_VERSION_INFO < (3, 5, 0):
-    FrameInfo = collections.namedtuple(
-        "FrameInfo",
-        ["frame", "filename", "lineno", "function", "code_context", "index"],
-    )
-
-    def getouterframes(frame, context=1):
-        """Wrapper for getouterframes so that it acts like the Python v3.5 version."""
-        return [FrameInfo(*f) for f in inspect.getouterframes(frame, context=context)]
-
-
-else:
-    getouterframes = inspect.getouterframes
-
-
-class Inspector(object):
+class Inspector:
     """Inspects objects."""
 
     def __init__(self, str_detail_level=0):
@@ -356,16 +336,16 @@ class Inspector(object):
         exception is suppressed.
         """
         try:
-            hdef = oname + inspect.signature(*getargspec(obj))
+            hdef = oname + str(inspect.signature(obj))
             return cast_unicode(hdef)
-        except:  # pylint:disable=bare-except
+        except Exception:
             return None
 
     def noinfo(self, msg, oname):
         """Generic message when no information is found."""
-        print("No %s found" % msg, end=" ")
+        print(f"No {msg} found", end=" ")
         if oname:
-            print("for %s" % oname)
+            print(f"for {oname}")
         else:
             print()
 
@@ -411,7 +391,7 @@ class Inspector(object):
             if init_ds is not None:
                 lines.append(head("Init docstring:"))
                 lines.append(indent(init_ds))
-        elif hasattr(obj, "__call__"):
+        elif callable(obj):
             call_ds = getdoc(obj.__call__)
             if call_ds:
                 lines.append(head("Call docstring:"))
@@ -428,7 +408,7 @@ class Inspector(object):
         linecache.checkcache()
         try:
             src = getsource(obj)
-        except:  # pylint:disable=bare-except
+        except Exception:
             self.noinfo("source", oname)
         else:
             print(src)
@@ -445,9 +425,9 @@ class Inspector(object):
         # is defined, as long as the file isn't binary and is actually on the
         # filesystem.
         if ofile.endswith((".so", ".dll", ".pyd")):
-            print("File %r is binary, not printing." % ofile)
+            print(f"File {ofile!r} is binary, not printing.")
         elif not os.path.isfile(ofile):
-            print("File %r does not exist, not printing." % ofile)
+            print(f"File {ofile!r} does not exist, not printing.")
         else:
             # Print only text files, not extension binaries.  Note that
             # getsourcelines returns lineno with 1-offset and page() uses
@@ -461,16 +441,16 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         out = []
         if title_width == 0:
             title_width = max(len(title) + 2 for title, _ in fields)
         for title, content in fields:
             title_len = len(title)
-            title = "{BOLD_RED}" + title + ":{NO_COLOR}"
+            title = "{BOLD_RED}" + title + ":{RESET}"
             if len(content.splitlines()) > 1:
                 title += "\n"
             else:
@@ -485,16 +465,16 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         out = []
         if title_width == 0:
             title_width = max(len(title) + 2 for title, _ in fields)
         for title, content in fields:
             title_len = len(title)
-            title = "{BOLD_RED}" + title + ":{NO_COLOR}"
+            title = "{BOLD_RED}" + title + ":{RESET}"
             if not isinstance(content, str) or len(content.splitlines()) > 1:
                 title += "\n"
             else:
@@ -515,9 +495,9 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         if HAS_PYGMENTS:
             rtn = self._format_fields_tokens(fields, title_width=title_width)
@@ -622,8 +602,8 @@ class Inspector(object):
         if isalias:
             if not callable(obj):
                 if len(obj) >= 2 and isinstance(obj[1], str):
-                    ds = "Alias to the system command:\n  {0}".format(obj[1])
-                else:  # pylint:disable=bare-except
+                    ds = f"Alias to the system command:\n  {obj[1]}"
+                else:
                     ds = "Alias: " + str(obj)
             else:
                 ds = "Alias to " + str(obj)
@@ -651,7 +631,7 @@ class Inspector(object):
         try:
             bclass = obj.__class__
             out["base_class"] = str(bclass)
-        except:  # pylint:disable=bare-except
+        except Exception:
             pass
 
         # String form, but snip if too long in ? form (full in ??)
@@ -665,7 +645,7 @@ class Inspector(object):
                         q.strip() for q in ostr.split("\n")
                     )
                 out[str_head] = ostr
-            except:  # pylint:disable=bare-except
+            except Exception:
                 pass
 
         if ospace:
@@ -674,7 +654,7 @@ class Inspector(object):
         # Length (for strings and lists)
         try:
             out["length"] = str(len(obj))
-        except:  # pylint:disable=bare-except
+        except Exception:
             pass
 
         # Filename where object was defined
@@ -754,8 +734,8 @@ class Inspector(object):
             # objects which use instance-customized docstrings.
             if ds:
                 try:
-                    cls = getattr(obj, "__class__")
-                except:  # pylint:disable=bare-except
+                    cls = obj.__class__
+                except Exception:
                     class_ds = None
                 else:
                     class_ds = getdoc(cls)
