@@ -1,18 +1,16 @@
 """Implements the xonsh (tab-)completion context parser.
 This parser is meant to parse a (possibly incomplete) command line.
 """
+
 import enum
 import os
 import re
 from collections import defaultdict
 from typing import (
     Any,
-    Dict,
     Generic,
-    List,
     NamedTuple,
     Optional,
-    Tuple,
     TypeVar,
     Union,
     cast,
@@ -49,7 +47,7 @@ class CommandContext(NamedTuple):
     The object containing the current command's completion context.
     """
 
-    args: Tuple[CommandArg, ...]
+    args: tuple[CommandArg, ...]
     """The arguments in the command"""
     arg_index: int  # ``-1`` if the cursor isn't in the command.
     """The current argument's index"""
@@ -116,7 +114,7 @@ class PythonContext(NamedTuple):
     """The cursor's index in the multiline code"""
     is_sub_expression: bool = False
     """Whether this is a sub expression (``@(...)``)"""
-    ctx: Optional[Dict[str, Any]] = None
+    ctx: Optional[dict[str, Any]] = None
     """Objects in the current execution context"""
 
     def __repr__(self):
@@ -145,7 +143,7 @@ class CompletionContext(NamedTuple):
     This will be ``None`` when we can't be completing python, e.g. ``echo $(<TAB>``.
     """
 
-    def with_ctx(self, ctx: Dict[str, Any]) -> "CompletionContext":
+    def with_ctx(self, ctx: dict[str, Any]) -> "CompletionContext":
         if self.python is not None:
             return self._replace(python=self.python._replace(ctx=ctx))
         return self
@@ -211,8 +209,7 @@ class Spanned(Generic[T]):
         expansion_obj: Union[
             "ExpandableObject", ExpansionOperation, Missing
         ] = Missing.MISSING,
-    ) -> "Spanned[T]":
-        ...
+    ) -> "Spanned[T]": ...
 
     @overload
     def replace(
@@ -225,8 +222,7 @@ class Spanned(Generic[T]):
         expansion_obj: Union[
             "ExpandableObject", ExpansionOperation, Missing
         ] = Missing.MISSING,
-    ) -> "Spanned[T2]":
-        ...
+    ) -> "Spanned[T2]": ...
 
     def replace(
         self,
@@ -256,7 +252,7 @@ class Spanned(Generic[T]):
         )
 
 
-Commands = Spanned[List[Spanned[CommandContext]]]
+Commands = Spanned[list[Spanned[CommandContext]]]
 ArgContext = Union[Spanned[CommandContext], Commands, Spanned[PythonContext]]
 
 ExpandableObject = Union[Spanned[CommandArg], ArgContext]
@@ -334,7 +330,8 @@ class CompletionContextParser:
         "LT",
         "GT",
         "RSHIFT",
-        "IOREDIRECT",
+        "IOREDIRECT1",
+        "IOREDIRECT2",
     }
     used_tokens |= io_redir_tokens
     artificial_tokens = {"ANY"}
@@ -354,7 +351,7 @@ class CompletionContextParser:
 
         self.error = None
         self.debug = debug
-        self.lexer = Lexer(tolerant=True)
+        self.lexer = Lexer(tolerant=True, pymode=False)
         self.tokens = tuple(self.used_tokens | self.artificial_tokens)
 
         yacc_kwargs = dict(
@@ -376,7 +373,7 @@ class CompletionContextParser:
         self,
         multiline_text: str,
         cursor_index: int,
-        ctx: Optional[Dict[str, Any]] = None,
+        ctx: Optional[dict[str, Any]] = None,
     ) -> Optional[CompletionContext]:
         """Returns a CompletionContext from a command line.
 
@@ -467,8 +464,7 @@ class CompletionContextParser:
                 if self.cursor_in_span(inner_span) or (
                     # the cursor is in a space-separated multi keyword.
                     # even if the cursor's at the edge, the keyword should be considered as a normal arg
-                    tok.value in ("and", "or")
-                    and self.cursor_in_span(outer_span)
+                    tok.value in ("and", "or") and self.cursor_in_span(outer_span)
                 ):
                     tok.type = "ANY"
 
@@ -535,7 +531,7 @@ class CompletionContextParser:
         |
         """
         if len(p) == 2:
-            spanned_args: List[Spanned[CommandArg]] = p[1]
+            spanned_args: list[Spanned[CommandArg]] = p[1]
             span = slice(spanned_args[0].span.start, spanned_args[-1].span.stop)
         else:
             # empty command
@@ -722,7 +718,7 @@ class CompletionContextParser:
 
     def p_args_many(self, p):
         """args : args arg"""
-        args: List[Spanned[CommandArg]] = p[1]
+        args: list[Spanned[CommandArg]] = p[1]
         new_arg: Spanned[CommandArg] = p[2]
         last_arg: Spanned[CommandArg] = args[-1]
 
@@ -937,7 +933,9 @@ class CompletionContextParser:
             sub_expr = cast(ArgContext, arg.expansion_obj)
 
             # this arg is a subcommand or multiple subcommands, e.g. `$(a && b)`
-            expanded_obj: Optional[ArgContext] = self.try_expand_span(sub_expr, new_span)  # type: ignore
+            expanded_obj: Optional[ArgContext] = self.try_expand_span(  # type: ignore
+                sub_expr, new_span
+            )
             if expanded_obj is None:
                 return None
             return self.sub_expression_arg(expanded_obj)
@@ -967,7 +965,9 @@ class CompletionContextParser:
             # the last command is expandable
             # if it were an `ExpansionOperation`, `try_expand` would caught it instead
             expandable = cast(ExpandableObject, python_context.expansion_obj)
-            expanded_command: Optional[ExpandableObject] = self.try_expand_right(expandable, new_span.stop)  # type: ignore
+            expanded_command: Optional[ExpandableObject] = self.try_expand_right(
+                expandable, new_span.stop
+            )  # type: ignore
 
             if (
                 expanded_command is not None
@@ -990,7 +990,7 @@ class CompletionContextParser:
 
     def handle_command_arg(
         self, arg: Spanned[CommandArg]
-    ) -> Tuple[CommandContext, Optional[Union[CommandContext, PythonContext]]]:
+    ) -> tuple[CommandContext, Optional[Union[CommandContext, PythonContext]]]:
         """Create a command context from an arg which contains the cursor.
         Also return the internal cursor context if it exists.
         `args`, `arg_index`, and `subcmd_opening` aren't set by this function
@@ -1105,7 +1105,7 @@ class CompletionContextParser:
 
     def process_string_segment(
         self, string: str, span: slice
-    ) -> Tuple[str, Optional[int]]:
+    ) -> tuple[str, Optional[int]]:
         """Process a string segment:
         1. Return a relative_cursor if it's inside the span (for ``Spanned.cursor_context``).
         2. Handle line continuations in the string.
