@@ -7,7 +7,6 @@ import importlib
 import os
 import sys
 import types
-import typing as tp
 from pathlib import Path
 from traceback import extract_tb, format_list
 from unittest.mock import MagicMock
@@ -35,14 +34,7 @@ def pytest_collection_modifyitems(items):
     """Move xsh test first to work around a bug in normal
     pytest cleanup. The order of tests are otherwise preserved.
     """
-    xsh_items = []
-    other_items = []
-    for item in items:
-        if isinstance(item, XshFunction):
-            xsh_items.append(item)
-        else:
-            other_items.append(item)
-    items[:] = xsh_items + other_items
+    items.sort(key=lambda item: -isinstance(item, XshFunction))
 
 
 def _limited_traceback(excinfo):
@@ -57,7 +49,7 @@ def _limited_traceback(excinfo):
         return format_list(tb)
 
 
-def pytest_collect_file(file_path: Path, path, parent):
+def pytest_collect_file(file_path: Path, parent):
     if file_path.suffix.lower() == ".xsh" and file_path.name.startswith("test_"):
         return XshFile.from_parent(parent, path=file_path)
 
@@ -129,7 +121,7 @@ def xonsh_execer_parse(xonsh_execer):
 
 @pytest.fixture
 def mock_executables_in(xession, tmp_path, monkeypatch):
-    def _factory(binaries: tp.List[str]):
+    def _factory(binaries: list[str]):
         xession.env["PATH"] = [tmp_path]
         exec_mock = MagicMock(return_value=binaries)
         monkeypatch.setattr(commands_cache, "executables_in", exec_mock)
