@@ -16,7 +16,7 @@ import typing as tp
 import xonsh.wizard as wiz
 from xonsh import __version__ as XONSH_VERSION
 from xonsh.built_ins import XSH
-from xonsh.cli_utils import Annotated, Arg, ArgParserAlias, add_args
+from xonsh.cli_utils import Arg, ArgParserAlias
 from xonsh.events import events
 from xonsh.foreign_shells import CANON_SHELL_NAMES
 from xonsh.lazyasd import lazyobject
@@ -561,6 +561,21 @@ def _info(
     data.extend([("xontrib", xontribs_loaded())])
     data.extend([("RC file", XSH.rc_files)])
 
+    # Show sensitive env variables that could affect the shell behavior.
+    envs = {
+        "UPDATE_OS_ENVIRON": None,
+        "XONSH_CAPTURE_ALWAYS": None,
+        "XONSH_SUBPROC_OUTPUT_FORMAT": None,
+        "THREAD_SUBPROCS": None,
+        "ENABLE_ASYNC_PROMPT": True,
+        "ENABLE_COMMANDS_CACHE": False,
+        "XONSH_CACHE_EVERYTHING": True,
+        "XONSH_CACHE_SCRIPTS": None,
+    }
+    for e, show_if in envs.items():
+        if (val := XSH.env.get(e)) is not None and (show_if is None or val == show_if):
+            data.extend([(e, val)])
+
     formatter = _xonfig_format_json if to_json else _xonfig_format_human
     s = formatter(data)
     return s
@@ -640,7 +655,7 @@ def xonfig_color_completer(*_, **__):
 
 
 def _colors(
-    style: Annotated[str, Arg(nargs="?", completer=xonfig_color_completer)] = None
+    style: tp.Annotated[str, Arg(nargs="?", completer=xonfig_color_completer)] = None,
 ):
     """Preview color style
 
@@ -693,7 +708,7 @@ def _web(
 
     from xonsh.webconfig import main
 
-    main.main(_args[1:])
+    main.serve(browser)
 
 
 class XonfigAlias(ArgParserAlias):
@@ -709,8 +724,7 @@ class XonfigAlias(ArgParserAlias):
     def build(self):
         parser = self.create_parser(prog="xonfig")
         # register as default action
-        add_args(parser, _info, allowed_params=())
-        parser.add_command(_info)
+        parser.add_command(_info, default=True)
         parser.add_command(_web)
         parser.add_command(_wizard)
         parser.add_command(_styles)
